/**
 *@file helpers.hpp some help functions
	@author: T. Hartmann and M. Bernt
 */
#ifndef _HELPERS_HPP_
#define _HELPERS_HPP_

#include <string>
#include <vector>
#include <iterator>
#include <set>

#include "genome.hpp"
using namespace std;

/**
 * string tokenizer
 * @param[in] str the string to split
 * @param[out] tokens the 'splits'
 * @param[in] delimiters the characters at which the string should be splitted (default ' ')
 */
void split_string(const string& str,
                      vector<string>& tokens,
                      const string& delimiters = " ");

/**
 * remove leading whitespace from a string
 * @param[in,out] str the string
 */
void strip_leading_ws(string &str);

/**
 * remove trailing whitespace from a string
 * @param[in,out] str the string
 */
void strip_trailing_ws(string &str);

/**
 * calculate decomposition into maximum increasing substrings and return in vector
 * @param[in] genome genome that has to be decomposed
 * @param[in,out] deco: ordered vector of sets each containing a maximum increasing substring
 */
void decomposition(genome g, vector< set<int> > &deco);

/**
 * calculates the minimum number of iTDRL needed to obtain a permutation from the identity permutation
 * @param[in] genome: target permutation
 * @return: minimum number of iTDRL
 */
unsigned int dist_iTDRL(genome g);

/**
 * calculates the number of maximal increasing substrings
 * @param[in] genome: permutation whos number of maximal increasing substrings shall be calculated
 * @return: number of maximal increasing substrings
 */
unsigned int max_inc_substrings(genome g);

/*
 * multiplication chromosomes of genomes
 * @param[in]: pi: first factor
 * @param[in]: sigma: second factor
 * @return: genome that contains as chromosome the product of pi°sigma; nmap and circ are inherited from sigma
 */
genome multiply(genome pi, genome sigma);

/*
 * calculates a sorting scenario to obtain a permutation from the identity permutation by only using iTDRLs
 * @param[in] g: target genome
 * @param[in/out] scen: scenario of genomes
 * @param[in/out] iTDRLs: iTDRL rrrmts that sort the genomes of scen
 * @param[in/out] last: last is true iff the last iTDRL is an left iTDRL
 */
void sort_by_iTDRL(genome g, vector<genome> &scen, vector<vector<set<int>>> &iTDRLs, bool &last);

/*
 * calculates a sorting scenario to obtain a permutation target from source by only using iTDRLs
 * @param[in] source: source genome
 * @param[in] target: target genome
 * @param[in/out] scen: scenario of genomes
 * @param[in/out] iTDRLs: iTDRL rrrmts that sort the genomes of scen
 * @param[in/out] last: last is true iff the last iTDRL is an left iTDRL
 */
void sort_by_iTDRL(genome source, genome target, vector<genome> &scen, vector<vector<set<int>>> &iTDRLs, bool &last);

/*
 * calulates for a given genome pi another genome sigma s.t. there exist an iTDRL rho with rho \circ sigma = pi
 * 	case 1 ( 0 is descent and n is ascent)
 * 	@param[in] g: genome pi
 * 	@param[in/out]: sigma such that the number of distance is "ceil (halved)"
 * 	@param[in/out] iTDRLs: vector of size two that contains in [0] (resp. [1]) all elements of sigma with the same (resp. opposite) sign
 *					iTDRLs is rho with iTDRL[0]=L and iTDRL[1]=R see the corresponding paper
 */
void sort_case_1(genome g, genome &sigma, vector<set<int>> &iTDRLs);

/*
 * calulates for a given genome pi another genome sigma s.t. there exist an iTDRL rho with rho \circ sigma = pi
 * 	case 2 ( n is descent and 0 is ascent)
 * 	@param[in] g: genome pi
 * 	@param[in/out]: sigma such that the number of distance is "ceil (halved)"
 * 	@param[in/out] iTDRLs: vector of size two that contains in [0] (resp. [1]) all elements of sigma with the same (resp. opposite) sign
 *					iTDRLs is rho with iTDRL[0]=L and iTDRL[1]=R see the corresponding paper
 */
void sort_case_2(genome g, genome &sigma, vector<set<int>> &iTDRLs);

/*
 * calculates the scenarios of all permutations of length size and checks whether or not these scenarios are correct
 * @param[in] size: length of the permutations that shall be tested
 * @param[in] numb: number of test that should be done
 */
void test_sorting_algorithm(unsigned int size, unsigned int numb);

/*
 * merges two sets first and second increasingly such that every element of second is inverted
 * @param[in/out] result: the resulting set containing all elements
 * @param[in] first: first set of integer
 * @para[in] second: second set of integer
 */
void oplus(set<int> &result, set<int> first, set<int> second);

#endif//_HELPERS_HPP_
